(document:surround "/std/frame")
(document:insert "/std/functions")

(document:envelop with-translation _ "alterator-standalone")

width 600
height 560

;;; ui replace

(define confirm-message (_ "There are unsaved changes in the active module.
Do you want to apply the changes before running the new module
or discard the changes?"))

;; possible states:
;; locked - you cannot leave this module
;; need-confirm - need confirmation from user
;; apply - apply changes and go to the next module
;; discard - discard changes and go to the next module
;; cancel - stay on this module
(define (ui-replace url status)
  (and url
       (case status
         ;;special values
         ((locked) (document:popup-information (_ "Module is locked") 'ok)
          (ui-replace url 'cancel))
         ((need-confirm) (ui-replace url (document:popup-warning confirm-message 'cancel 'apply 'discard)))
         ;;standard dialog return values
         ((cancel) (modules-id current (cell-ref *selected-item*)))
         ((apply) (if (frame:call-on-apply)
                      (ui-replace url 'discard)
                      (ui-replace url 'cancel)))
         ;;discard
         (else (frame:replace url)))))

;;; generate tree menu from available modules

(define *menu-items* (make-cell '()))

(define (make-search-item group)
  (lambda(item)
    (cons* group (cdr item) (car item))))

(define (init-groups)
  (fold-values (lambda(x menu-items search-items)
                 (let* ((group-tag (car x))
                        (group-name (cdr x))
                        (group-items (woo-list/name+label (string-append "/menu/" group-tag) 'ui "qt")))
                   (values
                    (append menu-items
                            (list group-name (map cdr group-items)))
                    (append search-items (map (make-search-item group-name) group-items)))))
               (list '() '())
               (woo-list/name+label "/menu" 'ui "qt")))

(define (init-menu)
  (let-values  ( ((menu-items search-items) (init-groups)) )
    (cell-set! *menu-items* search-items)
    (modules-id tree-items menu-items)))

;;; convert menu items to url

(define *selected-item* (make-cell '()))

(define (menu=? text)
  (lambda(x)
    (and (equal? text (list-head x 2))
         (cddr x))))

(define (selected-item)
  (let* ((text (modules-id tree-text))
         (text (if (= (length text) 1) (list (car text) (car text)) text)))
    (any (menu=? text) (cell-ref *menu-items*))))

;;; user interface

;;setup default globals
(frame:clear-on-leave)
(frame:clear-on-apply)

(splitbox columns "20;80"
          (frame:make-modules)
          (frame:make-page))

(document:root (when loaded
                 (woo-catch/message
                  (thunk
                   (init-menu)
                   (modules-id (when selected
                                 (ui-replace (selected-item) (frame:call-on-leave))
				 (cell-set! *selected-item* (modules-id current))))))))
